-- =============================================================================================================
-- *
-- * Copyright (c) M.Freeman
-- *
-- * File Name: addr_rom.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: Micro code ROM
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *   
-- =============================================================================================================

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY addr_rom IS
GENERIC(
  SIMULATION : BOOLEAN := TRUE;
  ADDR_WIDTH : INTEGER := 4;
  DATA_WIDTH : INTEGER := 8 );
PORT (
  addr: IN STD_LOGIC_VECTOR ( ADDR_WIDTH-1  DOWNTO 0 );
  data: OUT STD_LOGIC_VECTOR ( DATA_WIDTH-1 DOWNTO 0 )
);
END addr_rom;

ARCHITECTURE addr_rom_arch OF addr_rom IS

  CONSTANT INVALID_DATA : STD_LOGIC_VECTOR ((DATA_WIDTH-1) DOWNTO 0) := (OTHERS => 'X');
  
  subtype DATA_ROM_WORD IS STD_LOGIC_VECTOR ((DATA_WIDTH-1) DOWNTO 0);
  TYPE DATA_ROM_TABLE IS ARRAY (0 TO (2**ADDR_WIDTH)-1) OF DATA_ROM_WORD; 


  CONSTANT DATA_ROM: DATA_ROM_TABLE := DATA_ROM_TABLE'
  ( DATA_ROM_WORD'("00010000"), --0  
    DATA_ROM_WORD'("00000000"), --1	 			
    DATA_ROM_WORD'("00000000"), --2	 			
    DATA_ROM_WORD'("00000000"), --3	 			
    DATA_ROM_WORD'("00010001"), --4	 			
    DATA_ROM_WORD'("00000000"), --5	 			
    DATA_ROM_WORD'("00000000"), --6	 			
    DATA_ROM_WORD'("00000000"), --7	 			
    DATA_ROM_WORD'("00011000"), --8	 			
    DATA_ROM_WORD'("00010100"), --9	 			
    DATA_ROM_WORD'("00010010"), --10	 			
    DATA_ROM_WORD'("00000000"), --11	 			
    DATA_ROM_WORD'("00011001"), --12	 			
    DATA_ROM_WORD'("00000000"), --13	 			
    DATA_ROM_WORD'("00010101"), --14	 			
    DATA_ROM_WORD'("00000000")  --15	 			
  );
    
BEGIN
  PROCESS ( addr )
  BEGIN
    IF SIMULATION             
    THEN
      IF is_x( addr )                             -- Simulation version
      THEN
        data <= INVALID_DATA after 100 ps;
      ELSE
        data <= data_rom( CONV_INTEGER(addr)) after 100 ps;
      END IF;
    ELSE  
      data <= data_rom( CONV_INTEGER(addr));      -- Implementstion version
    END IF; 	
  END PROCESS;

END addr_rom_arch;
