-- =============================================================================================================
-- *
-- * Copyright (c) M.Freeman
-- *
-- * File Name: alu8bit.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description:
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes: Original design taken from web, a long time ago, unfortuently ive lost the URL
-- *        Mixed VHDL design styles using variables, boarding on the behavioural, however does
-- *        give a nice compact implementation
-- *
-- =============================================================================================================
--          
-- Interface
-- ---------
--
-- s2  s1  s0  cin  InputA InputB OutputF    Function       
-- 0   0   0   0      A      0     F=A       Transfer B     
-- 0   0   0   1      A      0     F=A+1     Increment A    
-- 0   0   1   0      A      B     F=A+B     Add B to A     
-- 0   0   1   1      A      B     F=A+B+1   Add B to A + 1 
-- 0   1   0   0      A  NOT B     F=A+-B    Add 1's Comp.  
-- 0   1   0   1      A  NOT B     F=A-B     Sub. B from A  
-- 0   1   1   0      A      1     F=A-1     Decrement A    
-- 0   1   1   1      A      1     F=A       Transfer A     
-- 1   0   0   X      A      B     F=AorB    OR             
-- 1   0   1   X      A      B     F=AxorB   XOR            
-- 1   1   0   X      A      B     F=AandB   AND            
-- 1   1   1   X      A      B     F=notA    NOT            
--

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

ENTITY alu8bit IS
PORT (
  a    : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
  b    : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
  cin  : IN  STD_LOGIC := '0';
  s2   : IN  STD_LOGIC := '0';
  s1   : IN  STD_LOGIC := '0';
  s0   : IN  STD_LOGIC := '0';
  f    : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
  cout : OUT STD_LOGIC
);
END alu8bit;

ARCHITECTURE alu8bit_arch OF alu8bit IS
BEGIN

  PROCESS(a, b, s2, s1, s0, cin)

    -- VARIABLES    
    VARIABLE temp_a     : STD_LOGIC_VECTOR(7 DOWNTO 0);
    VARIABLE temp_b     : STD_LOGIC_VECTOR(7 DOWNTO 0);
    VARIABLE sel        : STD_LOGIC_VECTOR(2 DOWNTO 0);
    VARIABLE temp_carry : STD_LOGIC_VECTOR(8 DOWNTO 0);
        
  BEGIN
    sel(2) := s1; 
    sel(1) := s0; 
    sel(0) := cin;
    
    IF (s2 = '0') 
    THEN
      CASE sel IS
        WHEN "000" => temp_a := "00000000"; temp_b := b;            -- F=B      Transfer B
        WHEN "001" => temp_a := a;          temp_b := "00000000";   -- F=A+1    Increment A
        WHEN "010" => temp_a := a;          temp_b := b;            -- F=A+B    Add B to A
        WHEN "011" => temp_a := a;          temp_b := b;            -- F=A+B+1  Add B to A + 1
        WHEN "100" => temp_a := a;          temp_b := not b;        -- F=A+-B   Add 1's Comp
        WHEN "101" => temp_a := a;          temp_b := not b;        -- F=A-B    Sub B from A
        WHEN "110" => temp_a := a;          temp_b := "11111111";   -- F=A-1    Decrement A
        WHEN "111" => temp_a := a;          temp_b := "11111111";   -- F=A      Transfer A  
        WHEN OTHERS => null;
      END CASE;  

      -- FULL ADDER

      temp_carry(0) := cin;
      FOR i IN 0 TO 7 LOOP 
        f(i) <= temp_a(i) xor temp_b(i) xor temp_carry(i);
        temp_carry(i+1) := (temp_a(i) and temp_b(i)) or (temp_a(i) and temp_carry(i)) or (temp_carry(i) and temp_b(i));
      END LOOP;
      cout <= temp_carry(8);
 
    ELSE

      -- LOGIC FUNCTIONS

      CASE sel IS
        WHEN "000" | "001" => f <= a or b;
        WHEN "010" | "011" => f <= a xor b;
        WHEN "100" | "101" => f <= a and b;
        WHEN "110" | "111" => f <= not a;
        WHEN OTHERS => null;
      END CASE;
      cout <= '0';
      
    END IF;
  END PROCESS;
END alu8bit_arch;
