-- =============================================================================================================
-- *
-- * Copyright (c) M.Freeman
-- *
-- * File Name: mcp_tcp.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: Mikes (minimal) CISC Processor testbench
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes: A very simple CISC processor implementation of the Von-Neumann processor discussed in lecture
-- *        slides. Simulation process base on PicoBlaze simulation process
-- *   
-- =============================================================================================================

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.ALL;
 
ENTITY mcp_tb IS
END mcp_tb;
 
ARCHITECTURE behavior OF mcp_tb IS 
 
  -- Component Declaration for the Unit Under Test (UUT)
 
  COMPONENT mcp_top_level is
  Port ( clk    : in  STD_LOGIC;
         clr    : in  STD_LOGIC;
         output : out STD_LOGIC_VECTOR (15 downto 0);
         sw     : in  STD_LOGIC_VECTOR(3 downto 0) );
  END COMPONENT;
  
  --Inputs
  SIGNAL clk  : STD_LOGIC := '0';
  SIGNAL clr  : STD_LOGIC := '0';
  SIGNAL sw   : STD_LOGIC_VECTOR(3 DOWNTO 0) := (OTHERS => '0'); 

  --Outputs
  SIGNAL output : STD_LOGIC_VECTOR(15 DOWNTO 0);

  -- Clock period definitions
  CONSTANT clk_period : time := 10 ns;
 
BEGIN
 
  -- Instantiate the Unit Under Test (UUT)
  uut: mcp_top_level PORT MAP (
    clk  => clk,
    clr  => clr,
    output => output,
    sw  => sw );
  
  -- Clock process definitions
  clk_process :PROCESS
  BEGIN
    clk <= '0';
    WAIT FOR clk_period/2;
    clk <= '1';
    WAIT FOR clk_period/2;
  END PROCESS;
 
  -- reset process definitions
  clr_process :PROCESS
  BEGIN
    clr <= '0';
    WAIT FOR clk_period * 10;
    clr <= '1';
    WAIT;
  END PROCESS;
  
  -- Stimulus process
  stim_proc: PROCESS
  BEGIN    
    sw <= "0000";
    WAIT;
  END PROCESS;

END;
