----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 26.09.2017 16:29:59
-- Design Name: 
-- Module Name: mcp_top_level - mcp_top_level_arch
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity mcp_top_level is
Port ( clk    : in  STD_LOGIC;
       clr    : in  STD_LOGIC;
       output : out STD_LOGIC_VECTOR (15 downto 0);
       pin    : out STD_LOGIC_VECTOR (15 downto 0);     
       sw     : in  STD_LOGIC_VECTOR(3 downto 0) );
end mcp_top_level;

architecture mcp_top_level_arch of mcp_top_level is

COMPONENT clock_divider IS
PORT ( 
  clk   : in STD_LOGIC;
  clr   : in STD_LOGIC;
  clock : out STD_LOGIC);
END COMPONENT;

COMPONENT mcp IS
PORT ( 
  clk  : IN STD_LOGIC;
  clr  : IN STD_LOGIC;
  addr : OUT STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  din  : IN  STD_LOGIC_VECTOR( 15 DOWNTO 0);
  dout : OUT STD_LOGIC_VECTOR( 15 DOWNTO 0);
  rw   : OUT STD_LOGIC );
END COMPONENT;

COMPONENT ram IS
GENERIC(
  ADDR_WIDTH : INTEGER := 8;
  DATA_WIDTH : INTEGER := 16 );
PORT (
  clk  : IN  STD_LOGIC;
  addr : IN  STD_LOGIC_VECTOR( ADDR_WIDTH-1 DOWNTO 0 );
  din  : IN  STD_LOGIC_VECTOR( DATA_WIDTH-1 DOWNTO 0 );
  dout : OUT STD_LOGIC_VECTOR( DATA_WIDTH-1 DOWNTO 0 );
  rw   : IN  STD_LOGIC );
END COMPONENT;

COMPONENT output_port IS
PORT(
  clk    : IN  STD_LOGIC;
  clr    : IN  STD_LOGIC;
  addr   : IN  STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  din    : IN  STD_LOGIC_VECTOR( 15 DOWNTO 0);
  output : OUT STD_LOGIC_VECTOR( 15 DOWNTO 0);
  rw     : IN  STD_LOGIC );
END COMPONENT;

SIGNAL address    : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
SIGNAL data_in    : STD_LOGIC_VECTOR( 15 DOWNTO 0 );
SIGNAL data_out   : STD_LOGIC_VECTOR( 15 DOWNTO 0 );
SIGNAL read_write : STD_LOGIC;

SIGNAL clk_INT    : STD_LOGIC;
SIGNAL clr_N      : STD_LOGIC;

SIGNAL output_int0  : STD_LOGIC_VECTOR( 15 DOWNTO 0 );
SIGNAL output_int1  : STD_LOGIC_VECTOR( 15 DOWNTO 0 );

begin

  clr_N <= not clr;
  
  pin <= output_int1;
  output <= output_int1; 
  
  output_int1(15 downto 4) <= output_int0(15 downto 4);
  
  output_int1(3) <= output_int0(3) or sw(3);
  output_int1(2) <= output_int0(2) or sw(2);
  output_int1(1) <= output_int0(1) or sw(1);
  output_int1(0) <= output_int0(0) or sw(0);
  
  CLK_DIV: clock_divider PORT MAP( 
    clk => clk,
    clr => clr_N,
    clock => clk_INT );
  
  CPU: mcp PORT MAP( 
    clk  => clk_INT,
    clr  => clr_N,
    addr => address,
    din  => data_in,
    dout => data_out,
    rw   => read_write);

  MEM: ram GENERIC MAP(
    ADDR_WIDTH => 8,
    DATA_WIDTH => 16 )
  PORT MAP(
    clk  => clk_INT,
    addr => address,
    din  => data_out,
    dout => data_in,
    rw   => read_write);
    
  GPO : output_port PORT MAP(
    clk    => clk_INT,
    clr    => clr_N,
    addr   => address,
    din    => data_out,
    output => output_int0,
    rw     => read_write );

end mcp_top_level_arch;
